--------------------------------------------------------------------------------------
-- Rematch_TSMPetValues - Extends Rematch's pet list by TSM market values
--------------------------------------------------------------------------------------
-- Rematch_TSMPetValues.lua
--
--[[

Rematch_TSMPetValues extends Rematch's pet list by TSM market values,
so you don't have to cage a pet to see its TSM item value.

You can browse through your whole pet collection and can choose which precious pet
is due to sell.

It uses TSM to gather the item value and displays the Mininum Buyout Value (DBMinBuyout)
and the Regional Market Value Average (DBRegionMarketAvg), but you can change that
in the configuration.

Setting the price source to 'DBMinBuyout/DBRegionMarketAvg', you see an alarm notice
in front of the price info that suggests selling a pet (+++ down to +) or 
buying a pet (--- down to -) on this server because the price is higher or lower than
on other regional servers.

No configuration needed, just install it and pet values will show up in Rematch,
but you can change the price sources and the alarm trigger value in the configuration.

Rematch_PetValues requires Rematch and TradeSkillMaster v3 and TradeSkillMaster_AuctionDB.

Thanks to Mandaeule for her inspirational idea to make this addon and to Bregs from BregsVid
on Youtube for his nice visual introduction video to this addon.

Author: Caraxe
All Rights Reserved

Version 0.1.3 - 2017.11.22
------------------------------------------------------------------------------------]]

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")

addon.METADATA = {
	NAME = GetAddOnMetadata(..., "Title"),
	VERSION = GetAddOnMetadata(..., "Version")
}

local defaults = {
	profile = {
	},
	global = {
		valA = "DBMinBuyout",
		valB = "DBRegionMarketAvg",
		valTH = "5000",
	},
}

local function GetOptions(uiType, uiName, appName)
	if appName == addonName then
		local options = {
			type = "group",
			name = addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")",
			args = {
				desc1 = {
					type = "description",
					order = 0,
					name = GetAddOnMetadata(addonName, "Notes"),
					fontSize = "medium",
				},
				desc1a = {
					type = "description",
					order = 0.05,
					name = "",
				},
				g1 = {
					type = "group",
					order = 1,
					guiInline = true,
					name = L["Price Sources"],
					args = {
				valAVal = {
					name = L["First Price Source"],
					desc = L["Choose a TSM price source"],
					type = "input",
					order = 0.1,
					width = "double",
					get = function()
						return addon.db.global.valA
						end,
					set = function(info, value)
						if value == "" then value = nil end
						addon.db.global.valA = value
					end,
				},
				desc2 = {
					type = "description",
					order = 0.2,
					name = L["Default is 'DBMinBuyout', one of '/tsm sources'"],
					width = "double",
				},
				valBVal = {
					name = L["Second Price Source"],
					desc = L["Choose a TSM price source"],
					type = "input",
					order = 0.3,
					width = "double",
					get = function()
						return addon.db.global.valB
						end,
					set = function(info, value)
						if value == "" then value = nil end
						addon.db.global.valB = value
					end,
				},
				desc3 = {
					type = "description",
					order = 0.4,
					name = L["Default is 'DBRegionMarketAvg', one of '/tsm sources'"],
					width = "double",
				},
				},
				},
				g2 = {
					type = "group",
					order = 2,
					name = L["Alerts"],
					guiInline = true,
					args = {
				valTHVal = {
					name = L["Alert Trigger"],
					desc = L["Choose the amount of gold to trigger the +/- notice"],
					type = "input",
					order = 0.5,
					width = "double",
					get = function()
						return addon.db.global.valTH
						end,
					set = function(info, value)
						if value == "" then value = nil end
						addon.db.global.valTH = value
					end,
				},
				desc4a = {
					type = "description",
					order = 0.6,
					fontSize = "medium",
					name = L["If the difference between the first and second market value is greater than this value, an + is printed in front of a price info, up to three times. Otherwise an - is printed. Defaults to 5000."],
					width = "full",
				},
				desc4b = {
					type = "description",
					order = 0.65,
					fontSize = "medium",
					name = L["Example: Set the first price source to 'DBMinBuyout' and the second to 'DBRegionMarketAvg'. For every + you can expect a greater chance to gain gold, if you sell this pet on this server. For every - you can expect to gain gold, if you buy this pet on this server and sell it on an other server."],
					width = "full",
				},
				},
				},
			},
		}
		return options
	end
end


-- hooked Function is Rematch:FillPetListButton(button, petID, forLoadout)
local OldRFPLB = nil
local function NewRFPLB(obj, button, petID, forLoadout)
	OldRFPLB(obj, button, petID, forLoadout)

	local idType
	if Rematch.GetIDType then
		idType = Rematch.GetIDType(nil, petID)
	else
		return
	end

	local _, speciesID, tsmSpeciesID, customName, priceInfo, level, name, rarity, fps, sps
	level = 1 -- doesn't matter
	rarity = 3 -- doesn't matter
	fps = 0
	sps = 0

	if idType=="pet" then
		speciesID, customName, level, _, _, _, _, name = C_PetJournal.GetPetInfoByPetID(petID)
		tsmSpeciesID = speciesID
	elseif idType=="species" then
		name = C_PetJournal.GetPetInfoBySpeciesID(petID)
		tsmSpeciesID = petID
	else
		return
	end

	if Rematch.petInfo.Fetch then
		local petInfo = Rematch.petInfo:Fetch(petID)
		rarity = petInfo.rarity or rarity
	end

	if TSMAPI then
		if addon.db.global.valA then
			local itemValue = TSMAPI:GetItemValue("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity), addon.db.global.valA)
			if itemValue then
				itemValue = floor(itemValue / COPPER_PER_GOLD) * COPPER_PER_GOLD
				local itemValueString = TSMAPI:MoneyToString(itemValue, "OPT_TRIM")
				priceInfo = itemValueString
				fps = floor(itemValue / COPPER_PER_GOLD)
			else
				priceInfo = "0"
			end
		end

		if addon.db.global.valB then
			local itemValue = TSMAPI:GetItemValue("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity), addon.db.global.valB)
			if itemValue then
				itemValue = floor(itemValue / COPPER_PER_GOLD) * COPPER_PER_GOLD
				local itemValueString = TSMAPI:MoneyToString(itemValue, "OPT_TRIM")
				priceInfo = priceInfo .. " / " .. itemValueString
				sps = floor(itemValue / COPPER_PER_GOLD)
			else
				priceInfo = priceInfo .. " / 0"
			end
		end
	end

	if (priceInfo == "0" or priceInfo == "0 / 0") then
		priceInfo = nil
	else
		if fps > 0 and sps > 0 and tonumber(addon.db.global.valTH) then
			if ((fps - sps) >= 0) then
				if ((fps - sps) >= (tonumber(addon.db.global.valTH) * 3)) then
					priceInfo = "+++ " .. priceInfo
				elseif ((fps - sps) >= (tonumber(addon.db.global.valTH) * 2)) then
					priceInfo = "++ " .. priceInfo
				elseif ((fps - sps) >= (tonumber(addon.db.global.valTH) * 1)) then
					priceInfo = "+ " .. priceInfo
				end
			end
			if ((sps - fps) > 0) then
				if ((sps - fps) > (tonumber(addon.db.global.valTH) * 3)) then
					priceInfo = "--- " .. priceInfo
				elseif ((sps - fps) > (tonumber(addon.db.global.valTH) * 2)) then
					priceInfo = "-- " .. priceInfo
				elseif ((sps - fps) > (tonumber(addon.db.global.valTH) * 1)) then
					priceInfo = "- " .. priceInfo
				end
			end
		end
	end

	if priceInfo then
		if customName then
			button.SubName:SetText(priceInfo .. ", " .. name)
		else
			button.SubName:SetText(priceInfo)
		end
		button.Name:SetHeight(21)
		button.SubName:Show()

		-- debug
		-- button.Name:SetText("p:" .. tsmSpeciesID .. ":" .. level .. ":" .. tostring(rarity))
	end
end

function addon:OnInitialize()
	addon.db = AceDB:New(addonName .. "DB", defaults, true)

	AceConfigRegistry:RegisterOptionsTable(addonName, GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
	addon.optionsFrame = optionsFrame
end

function addon:OnEnable()
	if Rematch and Rematch.FillPetListButton and Rematch.GetIDType then
		OldRFPLB = Rematch.FillPetListButton
		Rematch.FillPetListButton = NewRFPLB
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", L[" hooked into Rematch"])

		-- new default price source with 0.1.3
		if addon.db.global.fRun == nil or addon.db.global.fRun ~= addon.METADATA.VERSION then
			if addon.db.global.valA == 'DBMarket' and addon.db.global.valB == 'DBGlobalMarketAvg' then
				addon.db.global.valA = 'DBMinBuyout'
				addon.db.global.valB = 'DBRegionMarketAvg'
				print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", L[" default price sources changed to DBMinBuyout/DBRegionMarketAvg"])
				addon.db.global.fRun = addon.METADATA.VERSION
			end
		end
	else
		print("|cFF33FF99" .. addonName .. " (" .. addon.METADATA.VERSION .. ")|r:", L[" NOT hooked into Rematch"])
	end
end

-- EOF
